import RefreshIcon from "@mui/icons-material/Refresh";
import { Checkbox, FormControlLabel, IconButton, Tooltip } from "@mui/material";
import { DataGrid, GridColDef, GridRowSelectionModel } from "@mui/x-data-grid";
import React from "react";
import { InboxApi, InboxEntry } from "../api/InboxApi";
import { useAlert } from "../hooks/context_providers/AlertDialogProvider";
import { useLoadingMessage } from "../hooks/context_providers/LoadingMessageProvider";
import { useUnmatchedInboxEntriesCount } from "../hooks/UnmatchedInboxEntriesCountProvider";
import { AmountWidget } from "../widgets/AmountWidget";
import { AsyncWidget } from "../widgets/AsyncWidget";
import { DateWidget } from "../widgets/DateWidget";
import { MoneyMgrWebRouteContainer } from "../widgets/MoneyMgrWebRouteContainer";
import { NewMovementWidget } from "../widgets/NewMovementWidget";
import { UploadedFileWidget } from "../widgets/UploadedFileWidget";

export function InboxRoute(): React.ReactElement {
  const loadingMessage = useLoadingMessage();
  const alert = useAlert();

  const unmatched = useUnmatchedInboxEntriesCount();

  const [entries, setEntries] = React.useState<InboxEntry[] | undefined>();
  const [includeAttached, setIncludeAttached] = React.useState(false);

  const loadKey = React.useRef(1);

  const load = async () => {
    setEntries(await InboxApi.GetList(includeAttached));
  };

  const reload = async (skipEntries: boolean) => {
    try {
      loadingMessage.show("Refreshing the list of inbox entries...");
      unmatched.reload();
      if (!skipEntries) await load();
    } catch (e) {
      console.error("Failed to load list of inbox entries!", e);
      alert(`Failed to refresh the list of inbox entries! ${e}`);
    } finally {
      loadingMessage.hide();
    }
  };

  return (
    <MoneyMgrWebRouteContainer
      label={"Inbox"}
      actions={
        <span>
          <FormControlLabel
            checked={includeAttached}
            control={
              <Checkbox
                onChange={(_e, value) => {
                  setIncludeAttached(value);
                }}
              />
            }
            label="Include attached"
          />
          <Tooltip title="Refresh table">
            <IconButton onClick={() => reload(false)}>
              <RefreshIcon />
            </IconButton>
          </Tooltip>
        </span>
      }
    >
      <div style={{ display: "flex", flexDirection: "column", flex: 1 }}>
        <div style={{ flex: 1, display: "flex", flexDirection: "column" }}>
          <AsyncWidget
            loadKey={loadKey.current + String(includeAttached)}
            load={load}
            errMsg="Failed to load the content of inbox!"
            build={() => <InboxTable entries={entries!} onReload={reload} />}
          />
          <NewMovementWidget isInbox onCreated={() => reload(false)} />
        </div>
      </div>
    </MoneyMgrWebRouteContainer>
  );
}

function InboxTable(p: {
  entries: InboxEntry[];
  onReload: (skipEntries: boolean) => void;
}): React.ReactElement {
  const [rowSelectionModel, setRowSelectionModel] =
    React.useState<GridRowSelectionModel>([]);

  const columns: GridColDef<(typeof p.entries)[number]>[] = [
    {
      field: "time",
      headerName: "Date",
      width: 98 + 80,
      editable: true,
      type: "dateTime",
      valueGetter(_, m) {
        return new Date(m.time * 1000);
      },
      valueSetter(v, row) {
        row.time = Math.floor(v.getTime() / 1000);
        return row;
      },
      renderCell: (params) => {
        return <DateWidget time={params.row.time} />;
      },
    },
    {
      field: "label",
      headerName: "Label",
      flex: 3,
      editable: true,
      type: "string",
    },
    {
      field: "amount",
      headerName: "Amount",
      width: 110,
      editable: true,
      type: "number",
      align: "left",
      headerAlign: "left",
      renderCell: (params) => {
        if (params.row.amount)
          return <AmountWidget amount={params.row.amount} />;
        else return <i>Unspecified</i>;
      },
    },
    {
      field: "file",
      headerName: "File",
      editable: false,
      flex: 3,
      renderCell: (params) => {
        return <UploadedFileWidget file_id={params.row.file_id} />;
      },
    },
  ];

  return (
    <div style={{ flex: 1 }}>
      <DataGrid<InboxEntry>
        columns={columns}
        rows={p.entries}
        autoPageSize
        checkboxSelection
        initialState={{
          sorting: {
            sortModel: [{ field: "time", sort: "desc" }],
          },
          columns: {
            columnVisibilityModel: {
              checked: false,
            },
          },
        }}
        onRowSelectionModelChange={(newRowSelectionModel) => {
          setRowSelectionModel(newRowSelectionModel);
        }}
        rowSelectionModel={rowSelectionModel}
        processRowUpdate={async (n) => {
          try {
            return await InboxApi.Update(n);
          } catch (e) {
            console.error("Failed to update movement information!", e);
            alert(`Failed to update row! ${e}`);
            throw e;
          } finally {
            p.onReload(true);
          }
        }}
      />
    </div>
  );
}
